/*
Copyright (c) 2004,2005, Deryk Robosson <deryk@0x0a.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the Deryk Robosson nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections;
using System.IO;
using System.Xml;
using System.Xml.Serialization;


namespace EvoTemplate
{
	/// <summary>
	/// Provide an object for maintaining application configuration.	
	/// </summary>
	public class AppConfig
	{
		#region Private variables
		private string username;
		private string center;
		private string roll;
		private string updateServer;
		private System.Windows.Forms.TabAlignment tabs;
		private bool tips;
		public ArrayList plugins;
		#endregion

		#region Default Constructor
		/// <summary>
		/// AppConfig default constructor
		/// </summary>
		public AppConfig()
		{
			username = "";
			center = "";
			roll = "";
			updateServer = "";
			tabs = System.Windows.Forms.TabAlignment.Top;
			tips = true;
			plugins = new ArrayList();
		}
		#endregion

		#region Username
		/// <summary>
		/// Sets or gets the users username
		/// </summary>
		public string Username
		{
			get {
				return username;
			}
			set {
				username = value;
			}
		}
		#endregion
		
		#region Center
		/// <summary>
		/// Sets or gets the users center
		/// </summary>
		public string Center
		{
			get {
				return center;
			}
			set {
				center = value;
			}
		}
		#endregion
		
		#region Roll
		/// <summary>
		/// Sets or gets the users roll
		/// </summary>
		public string Roll
		{
			get {
				return roll;
			}
			set {
				roll = value;
			}
		}
		#endregion
		
		#region Tabs
		/// <summary>
		/// Sets or gets the tab control orientation
		/// </summary>
		public System.Windows.Forms.TabAlignment Tabs
		{
			get {
				return tabs;
			}
			set {
				tabs = value;
			}
		}
		#endregion
		
		#region Tips
		/// <summary>
		/// Sets or gets Tips property
		/// </summary>
		public bool Tips
		{
			get {
				return tips;
			}
			set {
				tips = value;
			}
		}
		#endregion
		
		#region Plugins
		/// <summary>
		/// Sets or gets the Plugins ArrayList
		/// </summary>
		public ArrayList Plugins
		{
			get {
				return plugins;
			}
			set {
				plugins = value;
			}
		}
		#endregion

		#region UpdateServer
		public string UpdateServer
		{
			get {
				return updateServer;
			}
			set {
				updateServer = value;
			}
		}
		#endregion

		#region Save/Load AppConfig
		public void SaveConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/config.dat";
			TextWriter writer = null;

			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
				writer = new StreamWriter(basePathConfig);
			
				serializer.Serialize(writer, this);
			}
			catch(Exception ex)
			{
				throw new Exception("Error writing config", ex);
			}
			finally
			{
				if(writer != null)
				{
					writer.Close();
				}
			}
		}

		
		public void LoadConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/config.dat";
			FileStream fs = null;

			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
				throw new Exception("Missing configuration");
			}
			else
			{
				FileInfo fi = new FileInfo(basePathConfig);
				if(fi.Length == 0)
				{
					throw new Exception("Empty configuration");
				}
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
				fs = new FileStream(basePathConfig, FileMode.Open);

				AppConfig ac = new AppConfig();
				ac = (AppConfig)serializer.Deserialize(fs);
				this.Username = ac.Username;
				this.Roll = ac.Roll;
				this.Center = ac.Center;
				this.Tabs = ac.Tabs;
				this.Tips = ac.Tips;
				this.Plugins = ac.Plugins;
				this.UpdateServer = ac.UpdateServer;
			}
			catch(Exception ex)
			{
				throw new Exception("Empty configuration", ex);
			}
			finally
			{
				if(fs != null)
				{
					fs.Close();
				}
			}
		}
		#endregion
	}


	/// <summary>
	/// Basic object for maintaining information related
	/// to an EvoTemplate plugin DLL
	/// </summary>
	public class PluginConfig
	{
		#region private variables
		private string filename;
		private string name;
		private string version;
		private string description;
		private bool configurable;
		private Type type;
		private object baseobject;
		private bool enabled;
		private string menuname;
		#endregion

		#region Default Constructor
		public PluginConfig()
		{
			this.filename = "";
			this.name = "";
			this.version = "";
			this.description = "";
			this.configurable = false;
			this.menuname = "";
		}
		#endregion

		#region Public Properties
		public string Filename
		{
			set {
				this.filename = value;
			}
			get {
				return this.filename;
			}
		}

		public string Name
		{
			set {
				this.name = value;
			}
			get {
				return this.name;
			}
		}

		public string Version
		{
			set {
				this.version = value;
			}
			get {
				return this.version;
			}
		}

		public string Description
		{
			set {
				this.description = value;
			}
			get {
				return this.description;
			}
		}

		public bool Configurable
		{
			set {
				this.configurable = value;
			}
			get {
				return this.configurable;
			}
		}
		
		public Type ObjectType
		{
			set {
				this.type = value;
			}
			get {
				return this.type;
			}
		}
		
		public object BaseObject
		{
			set {
				this.baseobject = value;
			}
			get {
				return this.baseobject;
			}
		}
		
		public bool Enabled
		{
			set {
				this.enabled = value;
			}
			get {
				return this.enabled;
			}
		}
		
		public string MenuName
		{
			set {
				this.menuname = value;
			}
			get {
				return this.menuname;
			}
		}
		#endregion
	}


	/// <summary>
	/// Macro object class for housing keyboard macro definitions
	/// </summary>
	public class Macro
	{
		#region Private variables
		private string macroString;
		private string macroKeyString;
		private int modifier;
		#endregion

		#region Default Constructor
		public Macro()
		{
			this.macroString = String.Empty;
			this.macroKeyString = String.Empty;
			this.modifier = 0; // none
		}
		#endregion

		#region Properties
		public string MacroString
		{
			get
			{
				return this.macroString;
			}
			set
			{
				this.macroString = value;
			}
		}

		public string KeyString
		{
			get
			{
				return this.macroKeyString;
			}
			set
			{
				this.macroKeyString = value;
			}
		}

		public int Modifier
		{
			get
			{
				return this.modifier;
			}
			set
			{
				this.modifier = value;
			}
		}
		#endregion
	}


	/// <summary>
	/// Macro configuration class
	/// </summary>
	[XmlInclude(typeof(Macro))]
	public class MacroConfig
	{
		#region Private variables
		private ArrayList macros;
		#endregion
		
		#region Default Constructor
		public MacroConfig()
		{
			this.macros = new ArrayList();
		}
		#endregion

		#region GetString method
		public string GetString(string key)
		{
			string retval = String.Empty;

			foreach(Macro mac in this.macros)
			{
				if(mac.KeyString.Equals(key))
				{
					retval = mac.MacroString;
				}
			}
			
			return retval;
		}
		#endregion

		#region Macros properties
		public ArrayList Macros
		{
			get
			{
				return this.macros;
			}
			set
			{
				this.macros = value;
			}
		}
		#endregion

		#region Save/Load AppConfig methods
		public void SaveConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/macros.dat";
			TextWriter writer = null;

			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(MacroConfig));
				writer = new StreamWriter(basePathConfig);
			
				serializer.Serialize(writer, this);
			}
			catch(Exception ex)
			{
				throw new Exception("Error writing config", ex);
			}
			finally
			{
				if(writer != null)
				{
					writer.Close();
				}
			}
		}


		public void LoadConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/macros.dat";
			FileStream fs = null;
			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
				throw new Exception("Missing configuration");
			}
			else
			{
				FileInfo fi = new FileInfo(basePathConfig);
				if(fi.Length == 0)
				{
					throw new Exception("Empty configuration");
				}
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(MacroConfig));
				fs = new FileStream(basePathConfig, FileMode.Open);

				MacroConfig mc = new MacroConfig();
				mc = (MacroConfig)serializer.Deserialize(fs);
				this.macros = mc.macros;
			}
			catch(Exception ex)
			{
				throw new Exception("Empty config", ex);
			}
			finally
			{
				if(fs != null)
				{
					fs.Close();
				}
			}
		}
		#endregion
	}

	/// <summary>
	/// Screen position configuration class
	/// </summary>
	public class ScreenPosition
	{
		#region Private variables
		private System.Drawing.Point pos;
		#endregion

		#region Default Constructor
		public ScreenPosition()
		{
			pos = new System.Drawing.Point(0, 0);
		}
		#endregion

		#region Public properties
		public System.Drawing.Point Position
		{
			get
			{
				return pos;
			}
			set
			{
				pos = value;
			}
		}

		public int X
		{
			get
			{
				return pos.X;
			}
			set
			{
				pos.X = value;
			}
		}

		public int Y
		{
			get
			{
				return pos.Y;
			}
			set
			{
				pos.Y = value;
			}
		}
		#endregion

		#region Save/Load AppConfig methods
		public void SaveConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/screenpos.dat";
			TextWriter writer = null;

			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(ScreenPosition));
				writer = new StreamWriter(basePathConfig);
			
				serializer.Serialize(writer, this);
			}
			catch(Exception ex)
			{
				throw new Exception("Error writing config", ex);
			}
			finally
			{
				if(writer != null)
				{
					writer.Close();
				}
			}
		}


		public void LoadConfig()
		{
			string basePathConfig = Directory.GetCurrentDirectory() + "/screenpos.dat";
			FileStream fs = null;

			if(!File.Exists(basePathConfig))
			{
				File.Create(basePathConfig);
				throw new Exception("Missing configuration");
			}
			else
			{
				FileInfo fi = new FileInfo(basePathConfig);
				if(fi.Length == 0)
				{
					throw new Exception("Empty configuration");
				}
			}

			try
			{
				XmlSerializer serializer = new XmlSerializer(typeof(ScreenPosition));
				fs = new FileStream(basePathConfig, FileMode.Open);

				ScreenPosition sp = new ScreenPosition();
				sp = (ScreenPosition)serializer.Deserialize(fs);
				this.pos = sp.Position;
			}
			catch(Exception ex)
			{
				throw new Exception("Empty config", ex);
			}
			finally
			{
				if(fs != null)
				{
					fs.Close();
				}
			}
		}
		#endregion
	}
}
